/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.robokind.demo.robot.replication;

import java.nio.ByteBuffer;
import java.util.List;
import java.util.Map;
import org.apache.avro.Schema;
import org.apache.avro.generic.GenericData.Array;
import org.jflux.api.core.Adapter;
import org.jflux.api.core.Source;
import org.jflux.api.core.event.Event;
import org.jflux.api.core.event.Header;
import org.jflux.api.core.event.MutableHeader.HeaderChanger;
import org.jflux.api.core.util.DefaultTimestampSource;
import org.jflux.avrogen.EventListRecord;
import org.jflux.avrogen.EventRecord;
import org.jflux.avrogen.HeaderRecord;

/**
 *
 * @author Matthew Stevenson
 */
public class EventEncoder<T> implements 
        Adapter<Event<Header<String,Long>,T>,EventRecord>{
    private Adapter<Header<String,Long>,HeaderRecord> myHeaderAdapter;
    private Adapter<T,ByteBuffer> myDataEncoder;
    
    public EventEncoder(Adapter<T,ByteBuffer> dataEncoder){
        myHeaderAdapter = 
                new HeaderChanger<String, Long, HeaderRecord>(
                        HeaderRecord.class, HeaderRecord.newBuilder());
        myDataEncoder = dataEncoder;
    }
    
    public void setDataEncoder(Adapter<T,ByteBuffer> dataEncoder){
        myDataEncoder = dataEncoder;
    }
    
    @Override
    public EventRecord adapt(Event<Header<String, Long>, T> a) {
        if(myDataEncoder == null){
            return null;
        }
        Header<String, Long> h = a.getHeader();
        T t = a.getData();
        HeaderRecord hRec = myHeaderAdapter.adapt(h);
        ByteBuffer b = myDataEncoder.adapt(t);
        EventRecord e = new EventRecord();
        e.setHeader(hRec);
        e.setData(b);
        return e;
    }
    
    public static class EventListEncoder implements 
            Adapter<List<EventRecord>,EventListRecord> {
        private final static Schema theEventArraySchema = Schema.createArray(HeaderRecord.SCHEMA$);
        private Source<HeaderRecord> myHeaderSource;

        public EventListEncoder(Source<HeaderRecord> headerSource){
            myHeaderSource = headerSource;
        }

        @Override
        public EventListRecord adapt(List<EventRecord> a) {
            if(a == null || a.isEmpty()){
                return null;
            }
            EventListRecord l = new EventListRecord();
            l.setHeader(myHeaderSource.getValue());
            l.setData(new Array<EventRecord>(theEventArraySchema, a));
            return l;
        }
    }
    
    public static class HeaderRecordSource implements Source<HeaderRecord> {
        private Source<HeaderRecord> myHeaderSource;
        private Source<Long> myTimestampSource;

        public HeaderRecordSource(
                String eventType, 
                String sourceReference, 
                Map<String,String> properties){
            HeaderRecord template = new HeaderRecord();
            template.setEventType(eventType);
            template.setSourceReference(sourceReference);
            template.setHeaderProperties(properties);
            myHeaderSource = HeaderRecord.newBuilder(template);
            myTimestampSource = new DefaultTimestampSource();
        }

        @Override
        public HeaderRecord getValue() {
            HeaderRecord h = myHeaderSource.getValue();
            h.setTimestamp(myTimestampSource.getValue());
            return h;
        }
    }
}
