/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.robokind.demo.robot.replication;

import org.jflux.api.core.Source;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.jflux.api.core.config.Configuration;
import org.jflux.api.core.util.ListMergeAdapter;
import org.osgi.framework.BundleContext;
import org.robokind.api.common.lifecycle.config.RKManagedGroupConfigUtils;
import org.robokind.api.common.lifecycle.utils.ManagedServiceFactory;
import org.robokind.api.common.lifecycle.utils.ManagedServiceGroup;
import org.robokind.api.messaging.services.ServiceCommand;
import org.robokind.demo.robot.replication.chain.NodeLinkLifecycleUtils;
import org.robokind.demo.robot.replication.lifecycle.EventBufferUtils;
import org.robokind.demo.robot.replication.lifecycle.ReplicationSenderGroupLifecycleUtils;
import org.robokind.impl.messaging.config.RKMessagingConfigUtils;
import static org.robokind.demo.robot.replication.lifecycle.ReplicationReceiverGroupLifecycleUtils.*;

/**
 *
 * @author matt
 */
public class ReplicationUtils {    
    public final static String BUFFER_LINK_ID_SUFFIX = "LinkId";
    public final static String ROUTE_LINK_ID_SUFFIX = "LinkId";
    public final static String DECODER_LINK_ID_SUFFIX = "DecoderId";
    
    private void initializeTest(BundleContext context, String groupId,
            String ipAddress, String destName, ManagedServiceFactory fact){
        String connectConfigId = groupId + "ConnectConfig";
        String destConfigId = groupId + "DestConfig";
        
        RKMessagingConfigUtils.registerConnectionConfig(
                connectConfigId, ipAddress, null, fact);
        RKMessagingConfigUtils.registerQueueConfig(destConfigId, destName, null, fact);
        
        startReplicator(context, groupId, destConfigId, connectConfigId, 1000L, 2000L, fact);
        startReceiver(groupId, connectConfigId, destConfigId, fact); 
        
        String commandBufferId = "commandBufferId";
        Class<ServiceCommand> commandClass = ServiceCommand.class;
        
        createReceiverRoute(groupId, commandBufferId, 
                "key", "id", commandClass, fact);  
        
        createReplicationBuffer(groupId, commandBufferId,
                "key", "id", 25, commandClass, fact);
    }
    
    public static ManagedServiceGroup startReplicator(
            BundleContext context, String replicationGroupId, 
            String destConfigId, String connectionConfigId, 
            long initialReplicationDelay, long replicationPeriod,
            ManagedServiceFactory fact){
        List<Configuration<String>> lifecycles = 
                getReplicatorLifecycleConfigs(context, replicationGroupId, 
                        initialReplicationDelay, replicationPeriod, 
                        destConfigId, connectionConfigId);
        Configuration<String> groupConf = 
                RKManagedGroupConfigUtils.buildManagedGroupConfig(
                        replicationGroupId, null, lifecycles);
        ManagedServiceGroup group = 
                RKManagedGroupConfigUtils.buildGroup(fact, groupConf);
        group.start();
        return group;
    }
    
    private static List<Configuration<String>> getReplicatorLifecycleConfigs(
            BundleContext context, String replicationGroupId, 
            long initialReplicationDelay, long replicationPeriod,
            String destConfigId, String connectionConfigId){
        return new ListMergeAdapter().adapt(Arrays.asList(
                ReplicationSenderGroupLifecycleUtils.getJMSLifecycleConfigs(
                        replicationGroupId, destConfigId, connectionConfigId),
                ReplicationSenderGroupLifecycleUtils.getReplicationSenderLifecycleConfigs(
                        replicationGroupId),
                ReplicationSenderGroupLifecycleUtils.getCollectorLifecycleConfigs(
                        replicationGroupId, context, null, 
                        initialReplicationDelay, replicationPeriod, null)
            ));
    }
    
    public static <T> ManagedServiceGroup createReplicationBuffer(
            String replicationGroupId, String bufferId, 
            String producerIdKey, String producerId, int bufferCapacity, 
            Class<T> messageClass, ManagedServiceFactory fact){
        List<Configuration<String>> configs = new ArrayList(
                ReplicationSenderGroupLifecycleUtils.getReplicationBufferLifecycleConfigs(
                        replicationGroupId, bufferId, 
                        messageClass.toString(), null, bufferCapacity));
        configs.add(NodeLinkLifecycleUtils.producerConsumerLinkLifecycleConfig(
                bufferId + BUFFER_LINK_ID_SUFFIX, null, 
                producerIdKey, producerId, null, 
                EventBufferUtils.PROP_EVENT_BUFFER_NODE_ID, bufferId, null));
        Configuration<String> groupConf = 
                RKManagedGroupConfigUtils.buildManagedGroupConfig(
                        replicationGroupId, null, configs);
        ManagedServiceGroup group = 
                RKManagedGroupConfigUtils.buildGroup(fact, groupConf);
        group.start();
        return group;
    }
    
    public static <T> ManagedServiceGroup createAlternateReplicationBuffer(
            String replicationGroupId, String bufferRouteKey, 
            Source<T> valueSource, Class<T> messageClass, 
            ManagedServiceFactory fact){
        List<Configuration<String>> configs = new ArrayList(
                ReplicationSenderGroupLifecycleUtils.getAlternateReplicationBufferLifecycleConfigs(
                        replicationGroupId, bufferRouteKey, 
                        messageClass.toString(), valueSource));
        Configuration<String> groupConf = 
                RKManagedGroupConfigUtils.buildManagedGroupConfig(
                        replicationGroupId, null, configs);
        ManagedServiceGroup group = 
                RKManagedGroupConfigUtils.buildGroup(fact, groupConf);
        group.start();
        return group;
    }
    
    
    public static <T> ManagedServiceGroup createReceiverRoute(
            String receiverGroupId, String routingKey, 
            String consumerIdKey, String consumerId, 
            Class<T> messageClass, ManagedServiceFactory fact) {
        List<Configuration<String>> configs = Arrays.asList(
                getReplicationRouteLinkLifecycleConfig(
                        receiverGroupId, routingKey + ROUTE_LINK_ID_SUFFIX,
                        routingKey, consumerIdKey, consumerId),
                getReplicationDecoderLinkLifecycleConfig(
                        receiverGroupId, routingKey + DECODER_LINK_ID_SUFFIX, 
                        messageClass.toString()));
        Configuration<String> groupConf = 
                RKManagedGroupConfigUtils.buildManagedGroupConfig(
                        receiverGroupId, null, configs);
        ManagedServiceGroup group = 
                RKManagedGroupConfigUtils.buildGroup(fact, groupConf);
        group.start();
        return group;
    }
    
    public static ManagedServiceGroup startReceiver(
            String receiverGroupId, String destConfigId, 
            String connectionConfigId, ManagedServiceFactory fact){
        List<Configuration<String>> lifecycles = 
                new ListMergeAdapter().adapt(Arrays.asList(
                    getJMSLifecycleConfigs(
                            receiverGroupId, destConfigId, connectionConfigId),
                    getReplicationReceiverLifecycleConfigs(receiverGroupId)
                ));
        Configuration<String> groupConf = 
                RKManagedGroupConfigUtils.buildManagedGroupConfig(
                        receiverGroupId, null, lifecycles);
        ManagedServiceGroup group = 
                RKManagedGroupConfigUtils.buildGroup(fact, groupConf);
        group.start();
        return group;
    }
}
