/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.robokind.demo.robot.replication.lifecycle;

import org.jflux.api.core.Source;
import java.util.List;
import java.util.Arrays;
import java.util.Collections;
import java.util.Map;
import org.jflux.api.core.Adapter;
import org.jflux.api.core.util.MapAdapter.MapValueAdapter;
import org.robokind.demo.robot.replication.EventListCollector.EventBufferTracker;
import org.osgi.framework.BundleContext;
import org.jflux.api.core.node.Node;
import java.util.Properties;
import java.util.concurrent.TimeUnit;
import org.jflux.api.core.Listener;
import org.jflux.api.data.blend.DataCollector;
import org.jflux.api.core.node.ProcessorNode;
import org.jflux.api.core.config.Configuration;
import org.jflux.api.core.node.ProducerNode;
import org.jflux.api.core.util.DefaultSource;
import org.jflux.api.schedule.HeartbeatNode;
import org.robokind.demo.robot.replication.EventListCollector;
import org.robokind.demo.robot.replication.chain.NodeLinkLifecycleUtils;

import static org.robokind.api.common.lifecycle.config.RKDependencyConfigUtils.*;
import static org.robokind.api.common.lifecycle.config.RKLifecycleConfigUtils.*;
import static org.robokind.api.common.lifecycle.config.RKManagedGroupConfigUtils.*;

/**
 *
 * @author Matthew Stevenson
 */
public class CollectorLifecycleUtils {
    private final static String theCollector = "encodeConfig";    
    
    public static List<Configuration<String>> buildTimedCollector(
            BundleContext context, String collectorIdKey, 
            String collectorId, Properties collectorProps,
            String trackerIdKey, String trackerId, String bufferFilterString,
            String senderLinkId, String senderNodeIdKey, String senderNodeId,
            String timerlinkId, String timerNodeIdKey, String timerNodeId,
            long initialDelay, long period, Properties timerProps){
        return Arrays.asList(
                buildEventCollectorLifecycleConfig(
                        collectorIdKey, collectorId, collectorProps),
                buildBufferTrackerLifecycleConfig(
                        context, collectorIdKey, collectorId, 
                        trackerIdKey, trackerId, bufferFilterString),
                buildSenderLinkLifecycleConfig(
                        senderLinkId, collectorIdKey, collectorId, 
                        senderNodeIdKey, senderNodeId),
                buildIntervalTimerLifecycleConfig(
                        timerNodeIdKey, timerNodeId, 
                        initialDelay, period, timerProps),
                buildCollectionTriggerLinkLifecycleConfig(
                        timerlinkId, timerNodeIdKey, timerNodeId, 
                        collectorIdKey, collectorId));
    }
    
    public static Configuration<String> buildEventCollectorLifecycleConfig(
            String collectorIdKey, String collectorId, Properties props){
        String[] classNames = new String[]{
                ProcessorNode.class.getName(), 
                EventListCollector.class.getName(),
                DataCollector.class.getName()};
        if(props == null){
            props = new Properties();
        }
        props.put(collectorIdKey, collectorId);
        return makeSimpleSelfBuildingLifecycle(
                classNames, EventListCollector.buildDefault(), 
                collectorIdKey, collectorId, props);
    }
    
    public static Configuration<String> buildBufferTrackerLifecycleConfig(
            final BundleContext context,
            String collectorIdKey, String collectorId,
            String trackerIdKey, String trackerId, 
            final String bufferFilterString){
        Configuration<String> depConf = buildLifecycleDependencyConfig(
                theCollector, EventListCollector.class, 
                collectorIdKey, collectorId, null, null);
        
        return makeSelfBuildingLifecycle(buildGenericLifecycleConfig(
                new String[]{EventBufferTracker.class.getName()}, 
                trackerIdKey, trackerId, null, Arrays.asList(depConf), 
                new MapValueAdapter(theCollector, 
                    new Adapter<EventListCollector,EventBufferTracker>() {
                        @Override
                        public EventBufferTracker adapt(EventListCollector a) {
                            EventBufferTracker t = new EventBufferTracker(
                                    context, bufferFilterString, a);
                            t.start();
                            return t;
                        }}),
                new Listener<EventBufferTracker>() {
                    @Override public void handleEvent(EventBufferTracker input) {
                        input.stop();
                    }}));
    }
    
    public static Configuration<String> buildSenderLinkLifecycleConfig(
            String linkId, String collectorIdKey, String collectorId, 
            String senderNodeIdKey, String senderNodeId){
        return NodeLinkLifecycleUtils.processorConsumerLinkLifecycleConfig(
                linkId, null, collectorIdKey, collectorId, null, 
                senderNodeIdKey, senderNodeId, null);
    }
    
    public static Configuration<String> buildCollectionTriggerLinkLifecycleConfig(
            String linkId, String producerNodeIdKey, String producerNodeId,
            String collectorIdKey, String collectorId){
        return NodeLinkLifecycleUtils.producerProcessorLinkLifecycleConfig(
                linkId, null, producerNodeIdKey, producerNodeId, null,
                collectorIdKey, collectorId, null);
    }
    
    public static Configuration<String> buildIntervalTimerLifecycleConfig(
            String timerIdKey, String timerId, 
            final long initialDelay, final long period, 
            final Source messageSource, Properties props){
        String[] classNames = new String[]{
                HeartbeatNode.class.getName(), 
                ProducerNode.class.getName(),
                Node.class.getName()};
        return makeSelfBuildingLifecycle(buildGenericLifecycleConfig(classNames, 
                timerIdKey, timerId, props, Collections.EMPTY_LIST, 
                new Adapter<Map<String, Object>, HeartbeatNode>() {
                    @Override public HeartbeatNode adapt(Map<String, Object> a) {
                        HeartbeatNode h = new HeartbeatNode(
                                (messageSource == null
                                        ? new DefaultSource(new Object()) 
                                        : messageSource),                                
                                initialDelay, period, TimeUnit.MILLISECONDS);
                        h.start();
                        return h;
                    }},
                new Listener<HeartbeatNode>() {
                    @Override public void handleEvent(HeartbeatNode input) {
                        input.stop();
                    }}));
    }
    
    public static Configuration<String> buildIntervalTimerLifecycleConfig(
            String timerIdKey, String timerId, 
            long initialDelay, long period, Properties props){
        return buildIntervalTimerLifecycleConfig(
                timerIdKey, timerId, initialDelay, period, null, props);
    }
}
